/*
#  Name:
#
#      PageMod (pagemod.js)
#
#  Description:
#
#      This module provides web browser page modification functions for visual 
#      enhancements and improved error handling.
#
#  Created by:
#
#      Walter Davidson (al2software@waltershobby.net)
#
#  Copyright 2016 Walter Davidson
#
#      Licensed under the Apache License, Version 2.0 (the "License");
#      you may not use this file except in compliance with the License.
#      You may obtain a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0
#
#      Unless required by applicable law or agreed to in writing, software
#      distributed under the License is distributed on an "AS IS" BASIS,
#      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#      See the License for the specific language governing permissions and
#      limitations under the License.
#
#  Change Log
#
#      February 17, 2016 - new
*/

/* Start script after HTML has been parsed and in a private name space */

document.addEventListener('DOMContentLoaded', function () {

/* Constants */

var WEBADDR = 'http://www.waltershobby.net/', 
    BACK404TEXT = 'Back to Top', 
    QUERY404 = '?e404', 
    ROOTQUERY404 = WEBADDR + QUERY404, 
    POPUPID = 'content-popup', 
    TOPANCHORIDPREFIX = 'top-list-anchor', 
    TOPBACKANCHORID = 'top-list-back-anchor', 
    ANCHORIDPREFIX = 'list-anchor', 
    BACKANCHORID = 'list-back-anchor', 
    CONTENTANCHORIDPREFIX = 'content-anchor', 
    RESRCANCHORIDPREFIX = 'resrc-anchor', 
    REDIRECTDELAY = 1750, 
    REDIRECTDELAYMIN = 4, 
    TARGETNEW = '_blank', 
    DEBUG = false;

/* State Variables */

var POPUP, 
    POPUPACTIVE = true, 
    URL = ['#'], 
    TIP = [''], 
    NOW = [true], 
    SELECTED = [false], 
    HOVERING = false, 
    REDIRECTINDEX = 0, 
    REDIRECTTIMER = undefined, 
    EVENTHANDLER = [];

function collectErrInfo(err) {
/* Gather and format error information

   Parameters:

       err:

           Error object

   Returns:

       errinfo:

           Formatted error information
*/

    var errinfo = '';
    try {
        if ('name' in err) {
            errinfo += err.name;
        } else {
            errinfo += 'Error';
        }
        if ('lineNumber' in err) {
            errinfo += ' (' + err.lineNumber + ')';
        }
        if ('message' in err) {
            errinfo += ':  ' + err.message;
        }
        if ('stack' in err) {
            errinfo += '\n---------------- Error Stack ----------------\n';
            try {
                errinfo += err.stack.trim();
            } catch (e) {
                errinfo += err.stack;
            }
        }
    } catch (e) {
        if ((!errinfo) || (errinfo == 'Error')) {
            return 'Error>  Not Available';
        }
    }
    return errinfo;
}

function logError(err) {
/* Logs error message

   Parameters:

       err:

           Error object
*/

    var errmsg = '';
    try {
        errmsg += '>>>>>>>  ' + WEBADDR + '  <<<<<<<';
        errmsg += '\nScript ' + collectErrInfo(err);
        console.log(errmsg);
    } catch (err2) {
        try {
            console.log(err);
            console.log(err2);
        } catch (e) {
            if (DEBUG) {
                alert(errmsg + '\n\nConsole Log ' + collectErrInfo(err2));
            }
        }
    }
}

function clearFocus(element) {
/* Removes focus from element on the page

   Parameters:

       element:

           Element to clear focus on or undefined to clear active element
*/

    var focused;
    if (element) {
        focused = element;
    } else {
        focused = document.activeElement;
    }
    if (focused && (focused !== document.body)) {
        if ('blur' in focused) {
            focused.blur();
        }
    }
}

function resetState() {
/* Resets page state
*/

    var i, j;
    clearFocus(undefined);
    hidePopup();
    for (i = 1, j = SELECTED.length; i < j; i++) {
        SELECTED[i] = false;
    }
    HOVERING = false;
}

function redirectBack() {
/* Redirects to previous page in the browser history
*/

    try {
        resetState();
        window.history.back();
    } catch (err) {
        logError(err);
    }
}

function redirectURL() {
/* Redirects to requested URL
*/

    try {
        resetState();
        REDIRECTTIMER = undefined;
        window.location.href = URL[REDIRECTINDEX];
    } catch (err) {
        logError(err);
    }
}

function redirectRequest(event) {
/*  Processes anchor selection

    Parameters:

        event:

            Event object
*/

    if (event.currentTarget.target == TARGETNEW) {
        clearFocus(event.target);
    } else {
        if (REDIRECTTIMER !== undefined) {
            clearTimeout(REDIRECTTIMER);
            REDIRECTTIMER = undefined;
        }
        resetState();
    }
    if (event.type != 'click') {
        event.preventDefault();
        event.target.click();
    }
}

function displayPopup(index, hover) {
/* Displays popup on the page

   Parameters:

       index:

           Index to text for selected anchor

       hover:

           Boolean indicating if hovering
*/

    if (hover) {
        HOVERING = true;
    }
    POPUP.innerHTML = TIP[index];
    POPUP.style.display = 'inherit';
    POPUPACTIVE = true;
}

function hidePopup() {
/* Hides popup on the page
*/

    if (POPUPACTIVE) {
        POPUP.style.display = 'none';
        POPUPACTIVE = false;
    }
}

function redirectPopup(index) {
/* Displays popup on the page (if needed) and redirects to new page

   Parameters:

       index:

           Index to selected anchor data
*/

    if (REDIRECTTIMER !== undefined) {
        clearTimeout(REDIRECTTIMER);
        REDIRECTTIMER = undefined;
    }
    if (URL[index] == ROOTQUERY404) {
        /* setTimeout needed for some browsers */
        setTimeout(redirectBack, REDIRECTDELAYMIN);
    } else {
        if (HOVERING || NOW[index] || SELECTED[index]) {
            REDIRECTINDEX = index;
            /* setTimeout needed for some browsers */
            setTimeout(redirectURL, REDIRECTDELAYMIN)
        } else {
            SELECTED[REDIRECTINDEX] = false;
            SELECTED[index] = true;
            displayPopup(index, false);
            REDIRECTINDEX = index;
            REDIRECTTIMER = setTimeout(redirectURL, REDIRECTDELAY);
        }
    }
}

function genEventDispatch(func, eventpassthru, arg1, arg2) {
/* Generates an event dispatch function

   Parameters:

       func:

           Function to be called

       eventpassthru:

           Boolean indicating to pass the event as the first argument to the 
           called function for processing (arg1 and arg2 will be ignored)

       arg1:

           First argument to pass to called function or undefined

       arg2:

           Second argument to pass to called function or undefined

   Returns:

       Dispatch function with error handling set up and default event actions
       canceled (if eventpassthru is false) prior to calling the function
*/

    if (eventpassthru) {
        return function (event) {
            try {
                func(event);
            } catch (err) {
                logError(err);
            }
        };
    } else {
        if (arg1 === undefined) {
            return function (event) {
                try {
                    event.preventDefault();
                    func();
                } catch (err) {
                    logError(err);
                }
            };
        } else if (arg2 === undefined) {
            return function (event) {
                try {
                    event.preventDefault();
                    func(arg1);
                } catch (err) {
                    logError(err);
                }
            };
        } else {
            return function (event) {
                try {
                    event.preventDefault();
                    func(arg1, arg2);
                } catch (err) {
                    logError(err);
                }
            };
        }
    }
}

function ignoreDrag(event) {
/* Cancels dragging

   Parameters:

       event:

           Event object
*/

    try {
        event.preventDefault();
        clearFocus(event.target);
    } catch (err) {
        logError(err);
    }
}

/* Initializes the page
*/

    try {
        var page404, i, j, k, element, back, idprefix;

/* Check browser capabilities (if insufficient, exit) */

        /* addEventListener above already checked for IE 9 or later */
        /* Check for Javascript 1.8 or later */
        if (!('reduce' in URL)) {
            return;
        }

/* Hide popup */

        POPUP = document.getElementById(POPUPID);
        hidePopup();
        
/* If 404 error page, return to previous page in the browser history if 404 
   was already encountered */

        element = document.getElementById(BACKANCHORID)
        if (element) {
            page404 = (
                (element.text == BACK404TEXT) && (window.history.length > 1)
            );
            if (page404) {
                if (
                    (window.location.search == QUERY404) && 
                    (document.referrer.indexOf(WEBADDR) == 0)
                ) {
                    /* setTimeout needed for some browsers */
                    setTimeout(redirectBack, REDIRECTDELAYMIN);
                    return;
                }
            }
        } else {
            page404 = false;
        }

/* Update anchors in main list and activate their events
   (if 404 error page, add query string) */

        EVENTHANDLER.push(
            genEventDispatch(hidePopup, false, undefined, undefined)
        );
        i = 1;
        k = 0;
        back = false;
        element = document.getElementById(ANCHORIDPREFIX + i);
        while (element) {
            if (page404) {
                URL.push(element.href + QUERY404);
            } else {
                URL.push(element.href);
            }
            TIP.push(element.title);
            element.removeAttribute('title');
            NOW.push(back);
            SELECTED.push(false);
            element.addEventListener('mouseout', EVENTHANDLER[0], false);
            EVENTHANDLER.push(genEventDispatch(displayPopup, false, i, true));
            element.addEventListener('mouseover', EVENTHANDLER[k + 1], false);
            EVENTHANDLER.push(
                genEventDispatch(redirectPopup, false, i, undefined)
            );
            k += 2;
            element.addEventListener('click', EVENTHANDLER[k], false);
            element.addEventListener('touchstart', EVENTHANDLER[k], false);
            if (back) {
                element = undefined;
            } else {
                i++;
                element = document.getElementById(ANCHORIDPREFIX + i);
                if (!element) {
                    element = document.getElementById(BACKANCHORID);
                    back = true;
                    if (page404 && (element)) {
                        element.title = 'Return to the previous page';
                        element.text = 'Back';
                    }
                }
            }
        }

/* Activate events for top list, content, and resource list anchors 
   (if 404 error page, add query string) */

        EVENTHANDLER.push(
            genEventDispatch(redirectRequest, true, undefined, undefined)
        );
        k++;
        for (j = 0; j < 3; j++) {
            if (j == 0) {
                idprefix = TOPANCHORIDPREFIX;
            } else if (j == 1) {
                if (page404) {
                    break;
                }
                idprefix = CONTENTANCHORIDPREFIX;
            } else {
                idprefix = RESRCANCHORIDPREFIX;
            }
            i = 1;
            back = false;
            element = document.getElementById(idprefix + i);
            while (element) {
                if (page404) {
                    element.href += QUERY404;
                }
                element.addEventListener('click', EVENTHANDLER[k], false);
                element.addEventListener('touchstart', EVENTHANDLER[k], false);
                if (back) {
                    element = undefined;
                } else {
                    i++;
                    element = document.getElementById(idprefix + i);
                    if (!element) {
                        if (j == 0) {
                            element = document.getElementById(TOPBACKANCHORID);
                            back = true;
                        }
                    }
                }
            }
        }

/* Disable dragging */

        document.addEventListener('dragstart', ignoreDrag, false);

/* Log error (if needed) */

    } catch (err) {
        logError(err);
    }

}, false);
